package internal

import (
	"context"
	"fmt"

	"github.com/go-redis/redis/v8"
	corev1 "k8s.io/api/core/v1"
)

type Manager struct {
	replicas map[string]string
	rdb      *redis.Client
}

func NewManager() *Manager {
	return &Manager{
		replicas: make(map[string]string),
		rdb: redis.NewClient(&redis.Options{
			Addr:     "localhost:6379",
			Password: "",
			DB:       0,
		}),
	}
}

func (m *Manager) AddReplica(ctx context.Context, pod *corev1.Pod) {
	_, err := m.rdb.Do(ctx, "replicaof", pod.Status.PodIP, "6379").Result()
	if err != nil {
		fmt.Printf("failed to add %s (%s)\n", pod.Status.PodIP, pod.Name)
		fmt.Println(err)
		return
	}
	m.replicas[pod.Name] = pod.Status.PodIP
	fmt.Printf("added %s (%s)\n", pod.Status.PodIP, pod.Name)
}

func (m *Manager) RemoveReplica(ctx context.Context, pod *corev1.Pod) {
	_, err := m.rdb.Do(ctx, "replicaof", "remove", pod.Status.PodIP, "6379").Result()
	if err != nil {
		fmt.Printf("failed to remove %s (%s)\n", pod.Status.PodIP, pod.Name)
		fmt.Println(err)
		return
	}
	delete(m.replicas, pod.Name)
	fmt.Printf("removed %s (%s)\n", pod.Status.PodIP, pod.Name)
}

func (m *Manager) GetReplica(name string) string {
	return m.replicas[name]
}
